﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BindingModel;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModel;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDataModels;
using SushiBarDataModels.Models;

namespace SushiBarBusinessLogic
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;

        public ShopLogic(ILogger logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }
        
        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}, ShopName:{ ShopName}", model?.Id, model?.Name);
            var list = model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadList. Id:{ Id}, ShopName:{ ShopName}", model.Id, model.Name);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if(_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Вставка в хранилище прервана");
                return false;
            }
            return true;
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if(_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Обновление прервано");
                return false;
            }
            return true;
        }

        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model);
            if(_shopStorage?.Delete(model) == null)
            {
                _logger.LogWarning("Удаление прервано");
                return false;
            }
            return true;
        }

        public bool AddSushiInShop(ShopSearchModel model, ISushiModel sushi, int count)
        {
            if (model == null) throw new ArgumentNullException(nameof(model));
            if(count <= 0)
                throw new ArgumentException(nameof(count));
            _logger.LogInformation("AddSushiInShop. ShopName:{ShopName}.Id:{ Id}", model.Name, model.Id);

            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("Не добавлено");
                return false;
            }
            if (element.ShopSushis.TryGetValue(sushi.Id, out var samesushi))
            {
                element.ShopSushis[sushi.Id] = (sushi, samesushi.Item2 + count);
                _logger.LogInformation("Same sushi found by supply. Added {0} of {1} in {2} shop", count, sushi.SushiName, element.Name);
            }
            else
            {
                element.ShopSushis[sushi.Id] = (sushi, count);
                _logger.LogInformation("New sushi added by supply. Added {0} of {1} in {2} shop", count, sushi.SushiName, element.Name);
            }

            _shopStorage.Update(new()
            {
                Id = element.Id,
                Name = element.Name,
                Address = element.Address,
                DateOpening = element.DateOpening,
                ShopSushis = element.ShopSushis
            });

            return true;
        }

        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if(model == null) 
                throw new ArgumentNullException($"{nameof(model)} является null");
            if (!withParams) return;
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет такого магазина", nameof(model.Name));
            }
            _logger.LogInformation("Shop. ShopName:{ShopName}.Address:{ Address}. Id:{ Id}", 
                model.Name, model.Address, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                Name = model.Name,
            });
            if(element != null && element.Id != model.Id && element.Name == model.Name)
            {
                throw new InvalidOperationException("Такой магазин с таким названием уже есть");
            }
        }
    }
}
